// Dart imports:
import 'dart:async';

// Flutter imports:
import 'package:flutter/material.dart';

// Project imports:
import 'package:zego_uikit/src/services/services.dart';

typedef MessageViewItemBuilder = Widget Function(
    BuildContext context, ZegoInRoomMessage message);

class ZegoInRoomMessageView extends StatefulWidget {
  final Stream<List<ZegoInRoomMessage>> stream;

  /// messages before stream broadcast
  final List<ZegoInRoomMessage> historyMessages;

  final MessageViewItemBuilder itemBuilder;

  const ZegoInRoomMessageView({
    Key? key,
    required this.stream,
    required this.itemBuilder,
    this.historyMessages = const [],
  }) : super(key: key);

  @override
  State<ZegoInRoomMessageView> createState() => _ZegoInRoomMessageViewState();
}

class _ZegoInRoomMessageViewState extends State<ZegoInRoomMessageView> {
  final ScrollController _scrollController = ScrollController();
  StreamSubscription<dynamic>? streamSubscription;
  var messagesNotifier = ValueNotifier<List<ZegoInRoomMessage>>([]);

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance?.addPostFrameCallback((_) {
      scrollDown();
    });

    streamSubscription = widget.stream.listen(onMessageUpdate);
  }

  @override
  void dispose() {
    super.dispose();
    streamSubscription?.cancel();
  }

  @override
  Widget build(BuildContext context) {
    messagesNotifier.value = widget.historyMessages;

    return ValueListenableBuilder<List<ZegoInRoomMessage>>(
      valueListenable: messagesNotifier,
      builder: (context, messageList, child) {
        return ListView.builder(
          shrinkWrap: true,
          controller: _scrollController,
          itemCount: messageList.length,
          itemBuilder: (context, index) {
            var message = messageList[index];

            return widget.itemBuilder.call(context, message);
          },
        );
      },
    );
  }

  void scrollDown() {
    _scrollController.animateTo(
      _scrollController.position.maxScrollExtent,
      duration: const Duration(milliseconds: 300),
      curve: Curves.easeOut,
    );
  }

  void onMessageUpdate(List<ZegoInRoomMessage> messages) {
    messagesNotifier.value = []; //  crazy bug
    messagesNotifier.value = messages;

    Future.delayed(const Duration(milliseconds: 100), () {
      scrollDown();
    });
  }
}
